/**
 * Copyright (c) 2011 ~ 2014 Deepin, Inc.
 *               2013 ~ 2014 jouyouyun
 *
 * Author:      jouyouyun <jouyouwen717@gmail.com>
 * Maintainer:  jouyouyun <jouyouwen717@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 **/

package accounts

import (
	"github.com/howeyc/fsnotify"
	"io/ioutil"
	"math/rand"
	"os"
	"path"
	dutils "pkg.linuxdeepin.com/lib/utils"
	"regexp"
	"strings"
)

type User struct {
	Uid            string
	Gid            string
	UserName       string
	HomeDir        string
	Shell          string
	IconFile       string
	BackgroundFile string
	AutomaticLogin bool
	AccountType    int32
	Locked         bool
	LoginTime      uint64
	HistoryIcons   []string
	IconList       []string
	objectPath     string
	watcher        *fsnotify.Watcher
	quitFlag       chan bool
}

func addUserToAdmList(name string) bool {
	tmps := []string{}
	tmps = append(tmps, "-a")
	tmps = append(tmps, name)
	tmps = append(tmps, "sudo")
	return execCommand(CMD_GPASSWD, tmps)
}

func deleteUserFromAdmList(name string) bool {
	tmps := []string{}
	tmps = append(tmps, "-d")
	tmps = append(tmps, name)
	tmps = append(tmps, "sudo")
	return execCommand(CMD_GPASSWD, tmps)
}

func getRandUserIcon() string {
	list := getIconList(ICON_SYSTEM_DIR)
	l := len(list)
	if l <= 0 {
		return ""
	}

	index := rand.Int31n(int32(l))
	return list[index]
}

func getIconList(dir string) []string {
	iconfd, err := os.Open(dir)
	if err != nil {
		logger.Warningf("Open '%s' failed: %v",
			dir, err)
		return nil
	}

	names, _ := iconfd.Readdirnames(0)
	list := []string{}
	for _, v := range names {
		if strings.Contains(v, "guest") {
			continue
		}

		tmp := strings.ToLower(v)
		//ok, _ := regexp.MatchString(`jpe?g$||png$||gif$`, tmp)
		ok1, _ := regexp.MatchString(`\.jpe?g$`, tmp)
		ok2, _ := regexp.MatchString(`\.png$`, tmp)
		ok3, _ := regexp.MatchString(`\.gif$`, tmp)
		if ok1 || ok2 || ok3 {
			list = append(list, path.Join(dir, v))
		}
	}

	return list
}

func getAdministratorList() []string {
	contents, err := ioutil.ReadFile(ETC_GROUP)
	if err != nil {
		logger.Errorf("ReadFile '%s' failed: %s", ETC_PASSWD, err)
		return nil
	}

	list := ""
	lines := strings.Split(string(contents), "\n")
	for _, line := range lines {
		strs := strings.Split(line, ":")
		if len(strs) != GROUP_SPLIT_LEN {
			continue
		}

		if strs[0] == "sudo" {
			list = strs[3]
			break
		}
	}

	return strings.Split(list, ",")
}

func setAutomaticLogin(name string) {
	dsp := getDefaultDisplayManager()
	logger.Debugf("Set %s Auto For: %s", name, dsp)
	switch dsp {
	case "lightdm":
		if dutils.IsFileExist(ETC_LIGHTDM_CONFIG) {
			dutils.WriteKeyToKeyFile(ETC_LIGHTDM_CONFIG,
				LIGHTDM_AUTOLOGIN_GROUP,
				LIGHTDM_AUTOLOGIN_USER,
				name)
		}
	case "gdm":
		if dutils.IsFileExist(ETC_GDM_CONFIG) {
			dutils.WriteKeyToKeyFile(ETC_GDM_CONFIG,
				GDM_AUTOLOGIN_GROUP,
				GDM_AUTOLOGIN_USER,
				name)
		}
	case "kdm":
		if dutils.IsFileExist(ETC_KDM_CONFIG) {
			dutils.WriteKeyToKeyFile(ETC_KDM_CONFIG,
				KDM_AUTOLOGIN_GROUP,
				KDM_AUTOLOGIN_ENABLE,
				true)
			dutils.WriteKeyToKeyFile(ETC_KDM_CONFIG,
				KDM_AUTOLOGIN_GROUP,
				KDM_AUTOLOGIN_USER,
				name)
		} else if dutils.IsFileExist(USER_KDM_CONFIG) {
			dutils.WriteKeyToKeyFile(ETC_KDM_CONFIG,
				KDM_AUTOLOGIN_GROUP,
				KDM_AUTOLOGIN_ENABLE,
				true)
			dutils.WriteKeyToKeyFile(USER_KDM_CONFIG,
				KDM_AUTOLOGIN_GROUP,
				KDM_AUTOLOGIN_USER,
				name)
		}
	default:
		logger.Warning("No support display manager")
	}
}

func isAutoLogin(username string) bool {
	dsp := getDefaultDisplayManager()
	//logger.Info("Display: ", dsp)

	switch dsp {
	case "lightdm":
		if dutils.IsFileExist(ETC_LIGHTDM_CONFIG) {
			v, ok := dutils.ReadKeyFromKeyFile(ETC_LIGHTDM_CONFIG,
				LIGHTDM_AUTOLOGIN_GROUP,
				LIGHTDM_AUTOLOGIN_USER,
				"")
			//logger.Info("AutoUser: ", v.(string))
			//logger.Info("UserName: ", username)
			if ok && v.(string) == username {
				return true
			}
		}
	case "gdm":
		if dutils.IsFileExist(ETC_GDM_CONFIG) {
			v, ok := dutils.ReadKeyFromKeyFile(ETC_GDM_CONFIG,
				GDM_AUTOLOGIN_GROUP,
				GDM_AUTOLOGIN_USER,
				"")
			if ok && v.(string) == username {
				return true
			}
		}
	case "kdm":
		if dutils.IsFileExist(ETC_KDM_CONFIG) {
			v, ok := dutils.ReadKeyFromKeyFile(ETC_KDM_CONFIG,
				KDM_AUTOLOGIN_GROUP,
				KDM_AUTOLOGIN_USER,
				"")
			if ok && v.(string) == username {
				return true
			}
		} else if dutils.IsFileExist(USER_KDM_CONFIG) {
			v, ok := dutils.ReadKeyFromKeyFile(USER_KDM_CONFIG,
				KDM_AUTOLOGIN_GROUP,
				KDM_AUTOLOGIN_USER,
				"")
			if ok && v.(string) == username {
				return true
			}
		}
	}

	return false
}

func getDefaultDisplayManager() string {
	contents, err := ioutil.ReadFile(ETC_DISPLAY_MANAGER)
	if err != nil {
		logger.Errorf("ReadFile '%s' failed: %s",
			ETC_DISPLAY_MANAGER, err)
		return ""
	}

	tmp := ""
	for _, b := range contents {
		if b == '\n' {
			tmp += ""
			continue
		}
		tmp += string(b)
	}

	return path.Base(tmp)
}

func (user *User) setProps() {
	info, _ := getUserInfoByPath(user.objectPath)
	user.setPropUserName(info.Name)
	user.setPropHomeDir(info.Home)
	user.setPropShell(info.Shell)
	user.setPropLocked(info.Locked)
	user.setPropAutomaticLogin(user.getPropAutomaticLogin())
	user.setPropAccountType(user.getPropAccountType())
	user.setPropIconList(user.getPropIconList())
	user.setPropIconFile(user.getPropIconFile())
	user.setPropBackgroundFile(user.getPropBackgroundFile())
	user.setPropHistoryIcons(user.getPropHistoryIcons())
}

func newUser(path string) *User {
	info, ok := getUserInfoByPath(path)
	if !ok {
		return nil
	}

	obj := &User{}
	obj.objectPath = info.Path
	obj.Uid = info.Uid
	obj.Gid = info.Gid

	var err error
	obj.watcher, err = fsnotify.NewWatcher()
	if err != nil {
		logger.Warning("New watcher in newUser failed:", err)
	}

	if obj.watcher != nil {
		obj.quitFlag = make(chan bool)
		obj.watchUserConfig()
		go obj.handUserConfigChanged()
	}

	return obj
}
